@@ -35,6 +35,7 @@
 #include <QLayout>
 #include <QProcess>
 #include <QPushButton>
+#include <QScreen>
 #include <QStringList>
 #include <QStyle>
 #include <QUrl>
@@ -44,10 +45,10 @@
 #include <KAuthObjectDecorator>
 #endif
 #include <KGuiItem>
+#include <KIconUtils>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KPageWidgetModel>
-#include <KIconLoader>
 
 bool KCMultiDialogPrivate::resolveChanges(KCModuleProxy *currentProxy)
 {
@@ -99,23 +100,8 @@ void KCMultiDialogPrivate::_k_slotCurrentPageChanged(KPageWidgetItem *current, K
         }
     }
 
-    // For some reason the KCModuleQml modules handle their own margin internally
-    // to match that we need to adjust the spacing of our page widget
-    // by removing the spacing between the left pane and the edge, and the layout on the right
-    // when we reduce the layout on the right we need to pad the buttons back to match
-
-    // TODO KF6 Fix this situation.
-
-    if (currentModule && currentModule->realModule() && currentModule->realModule()->inherits("KCModuleQml")) {
-        q->pageWidget()->layout()->setSpacing(0);
-        q->buttonBox()->setContentsMargins(q->style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
-                                           0,
-                                           q->style()->pixelMetric(QStyle::PM_LayoutRightMargin),
-                                           q->style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
-    } else {
-        q->buttonBox()->setContentsMargins(0, 0, 0, 0);
-        q->pageWidget()->layout()->setSpacing(q->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing));
-    }
+    // Remove margins for the buttonbox; KPageDialog handles that already
+    q->buttonBox()->setContentsMargins(0, 0, 0, 0);
 
     q->blockSignals(true);
     q->setCurrentPage(previous);
@@ -145,8 +131,10 @@ void KCMultiDialogPrivate::_k_clientChanged()
     }
 
     bool change = false;
+    bool defaulted = false;
     if (activeModule) {
         change = activeModule->changed();
+        defaulted = activeModule->defaulted();
 
         QPushButton *applyButton = q->buttonBox()->button(QDialogButtonBox::Apply);
         if (applyButton) {
@@ -154,7 +142,6 @@ void KCMultiDialogPrivate::_k_clientChanged()
 #ifndef KCONFIGWIDGETS_NO_KAUTH
             delete applyButton->findChild<KAuth::ObjectDecorator *>();
 #endif
-            applyButton->setEnabled(change);
         }
 
         QPushButton *okButton = q->buttonBox()->button(QDialogButtonBox::Ok);
@@ -194,26 +181,44 @@ void KCMultiDialogPrivate::_k_clientChanged()
 #endif
     }
 
+    auto buttons = activeModule->buttons();
+
     QPushButton *resetButton = q->buttonBox()->button(QDialogButtonBox::Reset);
     if (resetButton) {
+        resetButton->setVisible(buttons & KCModule::Apply);
         resetButton->setEnabled(change);
     }
 
     QPushButton *applyButton = q->buttonBox()->button(QDialogButtonBox::Apply);
     if (applyButton) {
+        applyButton->setVisible(buttons & KCModule::Apply);
         applyButton->setEnabled(change);
     }
 
-    if (activeModule) {
-        QPushButton *helpButton = q->buttonBox()->button(QDialogButtonBox::Help);
-        if (helpButton) {
-            helpButton->setEnabled(activeModule->buttons() & KCModule::Help);
-        }
+    QPushButton *cancelButton = q->buttonBox()->button(QDialogButtonBox::Cancel);
+    if (cancelButton) {
+        cancelButton->setVisible(buttons & KCModule::Apply);
+    }
 
-        QPushButton *defaultButton = q->buttonBox()->button(QDialogButtonBox::RestoreDefaults);
-        if (defaultButton) {
-            defaultButton->setEnabled(activeModule->buttons() & KCModule::Default);
-        }
+    QPushButton *okButton = q->buttonBox()->button(QDialogButtonBox::Ok);
+    if (okButton) {
+        okButton->setVisible(buttons & KCModule::Apply);
+    }
+
+    QPushButton *closeButton = q->buttonBox()->button(QDialogButtonBox::Close);
+    if (closeButton) {
+        closeButton->setHidden(buttons & KCModule::Apply);
+    }
+
+    QPushButton *helpButton = q->buttonBox()->button(QDialogButtonBox::Help);
+    if (helpButton) {
+        helpButton->setVisible(buttons & KCModule::Help);
+    }
+
+    QPushButton *defaultButton = q->buttonBox()->button(QDialogButtonBox::RestoreDefaults);
+    if (defaultButton) {
+        defaultButton->setVisible(buttons & KCModule::Default);
+        defaultButton->setEnabled(!defaulted);
     }
 }
 
@@ -226,7 +231,7 @@ void KCMultiDialogPrivate::_k_updateHeader(bool use, const QString &message)
     if (use) {
         item->setHeader(QStringLiteral("<b>") + kcm->moduleInfo().moduleName() + QStringLiteral("</b><br><i>") +
                         message + QStringLiteral("</i>"));
-        item->setIcon(KDE::icon(kcm->moduleInfo().icon(), QStringList() << QStringLiteral("dialog-warning")));
+        item->setIcon(KIconUtils::addOverlay(QIcon::fromTheme(kcm->moduleInfo().icon()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
     } else {
         item->setHeader(kcm->moduleInfo().moduleName());
         item->setIcon(QIcon::fromTheme(kcm->moduleInfo().icon()));
@@ -245,14 +250,17 @@ void KCMultiDialogPrivate::init()
                                   | QDialogButtonBox::RestoreDefaults
                                   | QDialogButtonBox::Cancel
                                   | QDialogButtonBox::Apply
+                                  | QDialogButtonBox::Close
                                   | QDialogButtonBox::Ok
                                   | QDialogButtonBox::Reset);
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), KStandardGuiItem::ok());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Cancel), KStandardGuiItem::cancel());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::RestoreDefaults), KStandardGuiItem::defaults());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Apply), KStandardGuiItem::apply());
+    KGuiItem::assign(buttonBox->button(QDialogButtonBox::Close), KStandardGuiItem::close());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Reset), KStandardGuiItem::reset());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Help), KStandardGuiItem::help());
+    buttonBox->button(QDialogButtonBox::Close)->setVisible(false);
     buttonBox->button(QDialogButtonBox::Reset)->setEnabled(false);
     buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 
@@ -305,9 +313,12 @@ void KCMultiDialog::showEvent(QShowEvent *ev)
      * We adjust the size after passing the show event
      * because otherwise window pos is set to (0,0)
      */
-    const QSize maxSize = QApplication::desktop()->availableGeometry(pos()).size();
-    resize(qMin(sizeHint().width(), maxSize.width()),
-           qMin(sizeHint().height(), maxSize.height()));
+    QScreen *screen = QApplication::screenAt(pos());
+    if (screen) {
+        const QSize maxSize = screen->availableGeometry().size();
+        resize(qMin(sizeHint().width(), maxSize.width()),
+                qMin(sizeHint().height(), maxSize.height()));
+    }
 }
 
 void KCMultiDialog::slotDefaultClicked()
@@ -498,7 +509,7 @@ KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo,
                 QStringLiteral("</b><br><i>") +
                 kcm->realModule()->rootOnlyMessage() +
                 QStringLiteral("</i>"));
-        item->setIcon(KDE::icon(moduleInfo.icon(), QStringList() << QStringLiteral("dialog-warning")));
+        item->setIcon(KIconUtils::addOverlay(QIcon::fromTheme(moduleInfo.icon()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
     } else {
         item->setHeader(moduleInfo.moduleName());
         item->setIcon(QIcon::fromTheme(moduleInfo.icon()));
@@ -513,7 +524,7 @@ KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo,
         const int siblingCount = model->rowCount(parentIndex);
         int row = 0;
         for (; row < siblingCount; ++row) {
-            KPageWidgetItem *siblingItem = model->item(parentIndex.child(row, 0));
+            KPageWidgetItem *siblingItem = model->item(model->index(row, 0, parentIndex));
             if (siblingItem->property("_k_weight").toInt() > moduleInfo.weight()) {
                 // the item we found is heavier than the new module
                 // qDebug() << "adding KCM " << item->name() << " before " << siblingItem->name();
